﻿//////////////////////////////////////////////
// Quaternion.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMaths
{
	class Matrix ;
}

/// Includes ---------------------------------

// nkGraphics
#include "Vector.h"

#include "../Dll/DllDefines.h"

/// Class ------------------------------------
	
namespace nkMaths
{
	class DLL_MATHS_EXPORT Quaternion : public Vector
	{
		public :
		
			// Constructor, destructor
			Quaternion () ;
			Quaternion (float x, float y, float z, float w) ;
			Quaternion (const Vector& axis, float angle) ;
			Quaternion (const Vector& eulerAngles) ;
			Quaternion (const Matrix& mat) ;
			Quaternion (const Quaternion& other) ;
			~Quaternion () ;

			// Getters
			Vector getAsEulerAngles () const ;

			// Setters
			Quaternion& setFromAxisAngle (const Vector& axis, float angle) ;
			Quaternion& setFromEuler (const Vector& angles) ;
			Quaternion& setFromOrientationMatrix (const Matrix& mat) ;
			Quaternion& setFromTransformationMatrix (const Matrix& mat) ;

			// Transformations
			Vector transform (const Vector& vector) const ;
			Vector& apply (Vector& vector) const ;

			// Operators
			// Assignment
			Quaternion& operator= (const Quaternion& other) ;
			// Mult
			Vector operator* (const Vector& other) const ;
			Quaternion operator* (const Quaternion& other) const ;
			void operator*= (const Quaternion& other) ;
	} ;
}