﻿//////////////////////////////////////////////
// Buffer.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkMemory
#include "../Dll/DllDefines.h"

// Standards
#include <cstring>
#include <vector>

/// Internals --------------------------------

namespace nkMemory
{
	struct BufferDataDescriptor
	{
		unsigned char* _data ;
		unsigned long long _size ;
	} ;
}

/// Class ------------------------------------

namespace nkMemory
{
	class DLL_MEMORY_EXPORT Buffer final
	{
		private :

			// Attributes
			unsigned char* _data ;
			unsigned long long _size ;

		public :

			// Functions
			// Constructors, destructor
			Buffer () ;
			Buffer (unsigned long long size) ;
			Buffer (unsigned char* data, unsigned long long size) ;
			Buffer (const Buffer& other) ;
			Buffer (Buffer&& other) ;
			~Buffer () ;

			// Getters
			unsigned char* getData () const ;
			unsigned long long getSize () const ;
			bool empty () const ;
			unsigned char& front () ;
			const unsigned char& front () const ;
			unsigned char& back () ;
			const unsigned char& back () const ;

			// Management
			nkMemory::Buffer& clear () ;
			nkMemory::Buffer& resize (unsigned long long size) ;
			nkMemory::Buffer& append (unsigned char value) ;
			nkMemory::Buffer& erase (unsigned long long index, unsigned long long count = 1ull) ;
			BufferDataDescriptor relinquishDataOwnership () ;

			// Operators
			unsigned char& operator[] (unsigned long long index) ;
			const unsigned char& operator[] (unsigned long long index) const ;

			Buffer& operator= (const Buffer& other) ;
			Buffer& operator= (Buffer&& other) ;

		public :

			// Constructors templated
			template <typename T>
			Buffer (const std::vector<T>& vec)
			:	Buffer((unsigned char*)vec.data(), vec.size() * sizeof(T))
			{
				// Nothing to do
			}

		public :

			// Memory ownership taking constructors
			static Buffer createAndTakeMemory (unsigned char* data, unsigned long long size) ;
	} ;
}